/*********************************************************************
 *
 *	Hardware specific definitions
 *
 *********************************************************************
 * FileName:        HardwareProfile.h
 * Dependencies:    None
 * Processor:       PIC18, PIC24F, PIC24H, dsPIC30F, dsPIC33F, PIC32
 * Compiler:        Microchip C32 v1.10 or higher
 *					Microchip C30 v3.12 or higher
 *					Microchip C18 v3.34 or higher
 *					HI-TECH PICC-18 PRO 9.63PL2 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright (C) 2002-2010 Microchip Technology Inc.  All rights
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and
 * distribute:
 * (i)  the Software when embedded on a Microchip microcontroller or
 *      digital signal controller product ("Device") which is
 *      integrated into Licensee's product; or
 * (ii) ONLY the Software driver source files ENC28J60.c, ENC28J60.h,
 *		ENCX24J600.c and ENCX24J600.h ported to a non-Microchip device
 *		used in conjunction with a Microchip ethernet controller for
 *		the sole purpose of interfacing with the ethernet controller.
 *
 * You should refer to the license agreement accompanying this
 * Software for additional information regarding your rights and
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author               Date		Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Howard Schlunder		10/03/06	Original, copied from Compiler.h
 * Boris Ribov          01/07/12    Modified for the PIC Etherfeed
 ********************************************************************/
#ifndef __HARDWARE_PROFILE_H
#define __HARDWARE_PROFILE_H

#include "GenericTypeDefs.h"
#include "Compiler.h"

// Set configuration fuses (but only once)
#if defined(THIS_IS_STACK_APPLICATION)
	#if defined(__18CXX)
		#if defined(__EXTENDED18__)
			#pragma config XINST=ON
		#elif !defined(HI_TECH_C)
			#pragma config XINST=OFF
		#endif
		
		//#if (defined(__18F97J60) || defined(__18F96J65) || defined(__18F96J60) || defined(__18F87J60) || defined(__18F86J65) || defined(__18F86J60) || defined(__18F67J60) || defined(__18F66J65) || defined(__18F66J60)) && !defined(HI_TECH_C)
		#pragma config WDT=OFF, FOSC2=ON, FOSC=HSPLL, ETHLED=ON
	#endif
#endif // Prevent more than one set of config fuse definitions


// Clock frequency value. This value is used to calculate Tick Counter value
#define GetSystemClock()		(41666667ul)      	// Hz
#define GetInstructionClock()	(GetSystemClock()/4)
#define GetPeripheralClock()	GetInstructionClock()


// Hardware mappings

//*****************************************************************************
// GPIO Ports assignments, configuration and initial default value
// Define the direction for each Input/Output pin (0-Output, 1-Input) and
// the initial state at the application startup
//
// PORTA Direction and initial status
//                     +-------------- n/a OSC1
//                     |+------------- n/a OSC2
//                     ||+------------ RA5 = output - XT6
//                     |||+----------- RA4 = output - XT5
//                     ||||+---------- RA3 = output - XT4
//                     |||||+--------- RA2 = output - XT3
//                     ||||||+-------- RA1 = output - Ethernet LED B
//                     |||||||+------- RA0 = output - Ethernet LED A
#define INIT_TRISA  (0b00000000)
#define INIT_PORTA  (0b00000000)

// PORTB Direction and initial status - INPUT; Data bus for the 8-bit LCD
//                     +-------------- RB7 
//                     |+------------- RB6 
//                     ||+------------ RB5
//                     |||+----------- RB4 
//                     ||||+---------- RB3
//                     |||||+--------- RB2
//                     ||||||+-------- RB1 
//                     |||||||+------- RB0
#define INIT_TRISB  (0b11111111)
#define INIT_PORTB  (0b00000000)

// PORTC Direction and initial status
//                     +-------------- RC7 = Input  - RS232
//                     |+------------- RC6 = Output - RS232
//                     ||+------------ RC5 = Used as SDO for SPI interface
//                     |||+----------- RC4 = Used as SDI for SPI interface
//                     ||||+---------- RC3 = Output - EEPROM CLK
//                     |||||+--------- RC2 = Output - EEPROM CS
//                     ||||||+-------- RC1 = Output - XT2
//                     |||||||+------- RC0 = Output - XT1
#define INIT_TRISC  (0b10010000)
#define INIT_PORTC  (0b00001000)

// PORTD Direction and initial status
//                     +-------------- RD7 = NA
//                     |+------------- RD6 = NA
//                     ||+------------ RD5 = NA
//                     |||+----------- RD4 = NA
//                     ||||+---------- RD3 = NA
//                     |||||+--------- RD2 = OUTPUT - DISPLAY CONTROL E
//                     ||||||+-------- RD1 = OUTPUT - DISPLAY CONTROL RW
//                     |||||||+------- RD0 = OUTPUT - DISPLAY CONTROL RS
#define INIT_TRISD  (0b00000000)
#define INIT_PORTD  (0b00000000)

// PORTE Direction and initial status
//                     +-------------- RE7 = NA
//                     |+------------- RE6 = NA
//                     ||+------------ RE5 = Output - XT13
//                     |||+----------- RE4 = Output - XT12
//                     ||||+---------- RE3 = Output - XT11
//                     |||||+--------- RE2 = Output - XT10
//                     ||||||+-------- RE1 = Ourput - XT9
//                     |||||||+------- RE0 = Output - XT8
#define INIT_TRISE  (0b00000000)
#define INIT_PORTE  (0b00000000)

// PORTF Direction and initial status
//                     +-------------- RF7 = Output - XT20
//                     |+------------- RF6 = Output - XT19
//                     ||+------------ RF5 = Output - XT18
//                     |||+----------- RF4 = Output - XT17
//                     ||||+---------- RF3 = Output	- XT16 
//                     |||||+--------- RF2 = Output - XT15
//                     ||||||+-------- RF1 = Output - XT14
//                     |||||||+------- RF0 = NA
#define INIT_TRISF  (0b00000000)
#define INIT_PORTF  (0b00000000)

// PORTG Direction and initial status
//                     +-------------- RG7 = NA
//                     |+------------- RG6 = NA
//                     ||+------------ RG5 = NA
//                     |||+----------- RG4 = Input - Reset button
//                     ||||+---------- RG3 = NA
//                     |||||+--------- RG2 = NA
//                     ||||||+-------- RG1 = NA
//                     |||||||+------- RG0 = NA 
#define INIT_TRISG  (0b00010000)
#define INIT_PORTG  (0b00000000)

// Initialization values for various registers
#define INIT_ADCON0 (0b00000000)     // ADON=0, Channel 0
#define INIT_ADCON1 (0b00001111)     // No analog inputs
#define INIT_ADCON2 (0b10111110)     // Right justify, 20TAD ACQ time, Fosc/64 (~21.0kHz)
//#define INIT_ADCON0 (0b00001001)   // ADON=0, Channel 2
//#define INIT_ADCON1 (0b00001011)   // AN0-AN3 are analog inputs

#define INIT_CMCON	(0b00000111)	 // Digital inputs, No comparators

// 25LC1024 I/O pins
#define EEPROM_CS_TRIS		(TRISCbits.TRISC2)
#define EEPROM_CS_IO		(LATCbits.LATC2)
#define EEPROM_SCK_TRIS		(TRISCbits.TRISC3)
#define EEPROM_SDI_TRIS		(TRISCbits.TRISC4)
#define EEPROM_SDO_TRIS		(TRISCbits.TRISC5)
#define EEPROM_SPI_IF		(PIR1bits.SSPIF)
#define EEPROM_SSPBUF		(SSPBUF)
#define EEPROM_SPICON1		(SSPCON1)
#define EEPROM_SPICON1bits	(SSPCON1bits)
#define EEPROM_SPICON2		(SSPCON2)
#define EEPROM_SPISTAT		(SSPSTAT)
#define EEPROM_SPISTATbits	(SSPSTATbits)

// RESET button on ICSP connector
#define	RESET_IO			(PORTGbits.RG4)

// LCD hardware signals
#define	LCD_I				(PORTBbits.RB7)
#define LCD_E				(LATDbits.LATD2)
#define LCD_RW				(LATDbits.LATD1)
#define LCD_RS				(LATDbits.LATD0)

// UART mapping functions for consistent API names across 8-bit and 16 or 
// 32 bit compilers.  For simplicity, everything will use "UART" instead of USART/EUSART/etc.
#define BusyUART()				BusyUSART()
#define CloseUART()				CloseUSART()
#define ConfigIntUART(a)		ConfigIntUSART(a)
#define DataRdyUART()			DataRdyUSART()
#define OpenUART(a,b,c)			OpenUSART(a,b,c)
#define ReadUART()				ReadUSART()
#define WriteUART(a)			WriteUSART(a)
#define getsUART(a,b,c)			getsUSART(b,a)
#define putsUART(a)				putsUSART(a)
#define getcUART()				ReadUSART()
#define putcUART(a)				WriteUSART(a)
#define putrsUART(a)			putrsUSART((far rom char*)a)


#endif