/************************************************************************

  	PIC Etherfeed - tickling your ego or statistics? If you have a
	website, blog or any other web service than the etherfeed is for you.
	The etherfeed is a small standalone network device, it connects to the
	Internet and displays statistical data about your website like number
	of unique visitors, number of pageviews, PayPal sales (SUM of sales via
	the PayPal IPN), Facebook likes for your brand / group and weather data
	like temperature, humidity for your location.
	
	OpenSource & OpenHardware project with kits available at:

	http://www.etherfeed.com
	https://www.facebook.com/etherfeed

    Copyright (C) 2012  Boris Ribov, Georgi Bakalski, Militsa Ilieva

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 *********************************************************************
 *
 *  Application based on Demo HTTP2 Server
 *  Support for HTTP2 module in Microchip TCP/IP Stack
 *	 -Implements the application 
 *	 -Reference: RFC 1002
 *
 * 	FileName: WEB.c
 *
 *********************************************************************
 * Boris Ribov   01/07/2012    Rev. 2.0		; Base version of the project
 ********************************************************************/
#define __CUSTOMHTTPAPP_C

#include "TCPIP Stack/TCPIP.h"
#include "Time.h"

#if defined(STACK_USE_HTTP2_SERVER)

// EEPROM configuration structure
extern APP_CONFIG AppConfig;			
// External variables for the time
extern BYTE SNTP_for_the_first_time; 	// Display if the NTP is firstly sync
extern DWORD upTime;					// Display the up time
extern DWORD sysTime;					// Display the system time
// Server script aviability flag
extern BYTE conn_flag;

/****************************************************************************
  Section:
	Function Prototypes and Memory Globalizers
  ***************************************************************************/
// Script post form
static HTTP_IO_RESULT HTTPPostScript(void);
// Config post form
static HTTP_IO_RESULT HTTPPostConfig(void);
// Factorys form
static HTTP_IO_RESULT HTTPPostDefault(void);
// Reboot button
static HTTP_IO_RESULT HTTPPostReboot(void);

// Sticky status message variable.
// This is used to indicated whether or not the previous POST operation was 
// successful.  The application uses these to store status messages when a 
// POST operation redirects.  This lets the application provide status messages
// after a redirect, when connection instance data has already been lost.
static BOOL lastSuccess = FALSE;

// Stick status message variable.  See lastSuccess for details.
static BOOL lastFailure = FALSE;

/****************************************************************************
  Section:
	Authorization Handlers
  ***************************************************************************/
  
/*****************************************************************************
  Function:
	BYTE HTTPNeedsAuth(BYTE* cFile)
	
  Internal:
  	See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/
BYTE HTTPNeedsAuth(BYTE* cFile)
{
	// If the filename begins with the folder "protect", then require auth
	if(memcmppgm2ram(cFile, (ROM void*)"protect", 7) == 0)
		return 0x00;		// Authentication will be needed later

	if(memcmppgm2ram(cFile, (ROM void*)"upload", 6) == 0)
		return 0x00;

	// You can match additional strings here to password protect other files.
	// You could switch this and exclude files from authentication.
	// You could also always return 0x00 to require auth for all files.
	// You can return different values (0x00 to 0x79) to track "realms" for below.

	return 0x80;			// No authentication required
}

/*****************************************************************************
  Function:
	BYTE HTTPCheckAuth(BYTE* cUser, BYTE* cPass)
	
  Internal:
  	See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/
BYTE HTTPCheckAuth(BYTE* cUser, BYTE* cPass)
{
	if(strcmppgm2ram((char *)cUser,(ROM char *)"admin") == 0
		&& strcmp((char *)cPass, (char *)AppConfig.Password) == 0)
		return 0x80;		// We accept this combination	

	return 0x00;			// Provided user/pass is invalid
}

/****************************************************************************
  Section:
	GET Form Handlers
  ***************************************************************************/
  
/*****************************************************************************
  Function:
	HTTP_IO_RESULT HTTPExecuteGet(void)
	
  Internal:
  	See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/
HTTP_IO_RESULT HTTPExecuteGet(void)
{
	BYTE *ptr;
	BYTE filename[20];
	
	// Load the file name
	// Make sure BYTE filename[] above is large enough for your longest name
	MPFSGetFilename(curHTTP.file, filename, 20);
	
	return HTTP_IO_DONE;
}

/****************************************************************************
  Section:
	POST Form Handlers
  ***************************************************************************/

/*****************************************************************************
  Function:
	HTTP_IO_RESULT HTTPExecutePost(void)
	
  Internal:
  	See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/
HTTP_IO_RESULT HTTPExecutePost(void)
{
	// Resolve which function to use and pass along
	BYTE filename[20];
	
	// Load the file name
	// Make sure BYTE filename[] above is large enough for your longest name
	MPFSGetFilename(curHTTP.file, filename, sizeof(filename));

	if(!memcmppgm2ram(filename, "protect/script.htm", 18))
		return HTTPPostScript();
	
	if(!memcmppgm2ram(filename, "protect/config.htm", 18))
		return HTTPPostConfig();

	if(!memcmppgm2ram(filename, "protect/def.htm", 15))
		return HTTPPostDefault();

	if(!memcmppgm2ram(filename, "protect/res.htm", 15))
		return HTTPPostReboot();

	return HTTP_IO_DONE;
}


/*****************************************************************************
  Function:
	static HTTP_IO_RESULT HTTPPostScript(void)

  Summary:
	Processes the configuration form on /protect/script.htm

  Description:
	Accepts configuration parameters from the form, saves them to a
	temporary location in RAM, then eventually saves the data to EEPROM or
	external Flash.
	
	When complete, this function redirects to protect/reboot.htm, which will
	then issue an AJAX call to reboot the device.

	This function creates a shadow copy of the AppConfig structure in 
	the upper portion of curHTTP.data, and then overwrites incoming data 
	there as it arrives.  For each name/value pair, the name is first 
	read to curHTTP.data[0:7].  Next, the value is read to 
	curHTTP.data[8:31].  The name is matched against an expected list, 
	and then the value is parsed and stored in the appropriate location 
	of the shadow copy of AppConfig.  Once all data has been read, the new
	AppConfig is saved back to EEPROM and the browser is redirected to 
	reboot.htm.  That file includes an AJAX call to reboot.cgi, which 
	performs the actual reboot of the machine.
	
	If some params cannot be parsed, the browser redirects to
	script_error.htm and does not save anything so as to prevent errors.

  Precondition:
	None

  Parameters:
	None

  Return Values:
  	HTTP_IO_DONE - all parameters have been processed
  	HTTP_IO_WAITING - the function is pausing to continue later
  	HTTP_IO_NEED_DATA - data needed by this function has not yet arrived
  ***************************************************************************/
static HTTP_IO_RESULT HTTPPostScript(void)
{
	APP_CONFIG *app;
	BYTE *ptr;
	WORD len;
	unsigned long refresh_val;
	
	#define SM_SRC_START		(0u)
	#define SM_SRC_READ_NAME	(1u)
	#define SM_SRC_READ_VALUE	(2u)
	#define SM_SRC_SUCCESS		(3u)
	#define SM_SRC_FAILURE		(4u)
	
	// Set app config pointer to use data array
	// if APP structure is increasing, then increase the size of curHTTP.data[] buffer
	app = (void*)(&curHTTP.data[32]); 
	
	switch(curHTTP.smPost)
	{
		case SM_SRC_START:
			// Use current config as defaults
			memcpy((void*)app, (void*)&AppConfig, sizeof(AppConfig));
			curHTTP.smPost = SM_SRC_READ_NAME;
			
		case SM_SRC_READ_NAME:
			// If all parameters have been read, end
			if(curHTTP.byteCount == 0)
			{
				curHTTP.smPost = SM_SRC_SUCCESS;
				break;
			}	
			// Read a name
			if(HTTPReadPostName(curHTTP.data, 8) == HTTP_READ_INCOMPLETE)
				return HTTP_IO_NEED_DATA;
			// Move to reading a value, but no break
			curHTTP.smPost = SM_SRC_READ_VALUE;
			
		case SM_SRC_READ_VALUE:
			// Read a value
			if(HTTPReadPostValue(curHTTP.data + 8, 24) == HTTP_READ_INCOMPLETE)
				return HTTP_IO_NEED_DATA;			
			// Default action after this is to read the name, unless there's an error
			curHTTP.smPost = SM_SRC_READ_NAME;
				
			// Parse the value that was read
			if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"addr"))
			{// Read the script server address
				for(len = strlen((char*)&curHTTP.data[8]); len < 20; len++)
					curHTTP.data[8+len] = 0;
				memcpy((void*)app->Scr_Svr, (void*)curHTTP.data+8, 20);
				app->Scr_Svr[20] = 0;
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"path"))
			{// Read the script server path
				for(len = strlen((char*)&curHTTP.data[8]); len < 20; len++)
					curHTTP.data[8+len] = 0;
				memcpy((void*)app->Scr_Path, (void*)curHTTP.data+8, 20);
				app->Scr_Path[20] = 0;
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ref"))
			{// Read the script refresh time
				refresh_val = atoul((char*)&curHTTP.data[8]);

				if ((refresh_val >= 10) && (refresh_val <= 60*60*24*31))
					app->Scr_Refresh = refresh_val;
				else
					curHTTP.smPost = SM_SRC_FAILURE;
			}
			break;
			
		case SM_SRC_SUCCESS:
			// Save new settings
			ptr = (BYTE*)app;
		    XEEBeginWrite(0x0000);
		    XEEWrite(0x61);
		    for (len = 0; len < sizeof(AppConfig); len++ )
		        XEEWrite(*ptr++);
		    XEEEndWrite();
	        while(XEEIsBusy());
		
			// Set the board to reboot to the new address
			strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/protect/reboot.htm?");
			memcpy((void*)(curHTTP.data+20), (void*)app->NetBIOSName, 16);
			ptr = curHTTP.data;
			while(*ptr != ' ' && *ptr != '\0')
				ptr++;
			*ptr = '\0';

			curHTTP.httpStatus = HTTP_REDIRECT;			
			return HTTP_IO_DONE;
			
		case SM_SRC_FAILURE:
			// An error occurred parsing the refresh time, so don't save 
			// anything to prevent errors.		
			strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/protect/script_error.htm");
			curHTTP.httpStatus = HTTP_REDIRECT;		
			return HTTP_IO_DONE;			
	}

	return HTTP_IO_WAITING;		// Assume we're waiting to process more data
}


/*****************************************************************************
  Function:
	static HTTP_IO_RESULT HTTPPostConfig(void)

  Summary:
	Processes the configuration form on protect/config.htm

  Description:
	Accepts configuration parameters from the form, saves them to a
	temporary location in RAM, then eventually saves the data to EEPROM or
	external Flash.
	
	When complete, this function redirects to protect/reboot.htm, which will
	then issue an AJAX call to reboot the device.

	This function creates a shadow copy of the AppConfig structure in 
	the upper portion of curHTTP.data, and then overwrites incoming data 
	there as it arrives.  For each name/value pair, the name is first 
	read to curHTTP.data[0:7].  Next, the value is read to 
	curHTTP.data[8:31].  The name is matched against an expected list, 
	and then the value is parsed and stored in the appropriate location 
	of the shadow copy of AppConfig.  Once all data has been read, the new
	AppConfig is saved back to EEPROM and the browser is redirected to 
	reboot.htm.  That file includes an AJAX call to reboot.cgi, which 
	performs the actual reboot of the machine.
	
	If an IP address cannot be parsed, the browser redirects to
	config_error.htm and does not save anything so as to prevent errors.

  Precondition:
	None

  Parameters:
	None

  Return Values:
  	HTTP_IO_DONE - all parameters have been processed
  	HTTP_IO_WAITING - the function is pausing to continue later
  	HTTP_IO_NEED_DATA - data needed by this function has not yet arrived
  ***************************************************************************/
static HTTP_IO_RESULT HTTPPostConfig(void)
{
	APP_CONFIG *app;
	BYTE *ptr;
	WORD len;
	
	#define SM_CFG_START		(0u)
	#define SM_CFG_READ_NAME	(1u)
	#define SM_CFG_READ_VALUE	(2u)
	#define SM_CFG_SUCCESS		(3u)
	#define SM_CFG_FAILURE		(4u)
	
	// Set app config pointer to use data array
	//if APP structure is increasing, then increase the size of curHTTP.data[] buffer
	app = (void*)(&curHTTP.data[32]); 
	
	switch(curHTTP.smPost)
	{
		case SM_CFG_START:
			// Use current config as defaults
			memcpy((void*)app, (void*)&AppConfig, sizeof(AppConfig));
			app->Flags.bIsDHCPEnabled = 0;		
			// Move to next state, but no break
			curHTTP.smPost = SM_CFG_READ_NAME;
			
		case SM_CFG_READ_NAME:
			// If all parameters have been read, end
			if(curHTTP.byteCount == 0)
			{
				curHTTP.smPost = SM_CFG_SUCCESS;
				break;
			}	
			// Read a name
			if(HTTPReadPostName(curHTTP.data, 8) == HTTP_READ_INCOMPLETE)
				return HTTP_IO_NEED_DATA;
			// Move to reading a value, but no break
			curHTTP.smPost = SM_CFG_READ_VALUE;
			
		case SM_CFG_READ_VALUE:
			// Read a value
			if(HTTPReadPostValue(curHTTP.data + 8, 24) == HTTP_READ_INCOMPLETE)
				return HTTP_IO_NEED_DATA;			
			// Default action after this is to read the name, unless there's an error
			curHTTP.smPost = SM_CFG_READ_NAME;
				
			// Parse the value that was read
			if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ip"))
			{// Read new static IP Address
				if(StringToIPAddress(curHTTP.data+8, &(app->MyIPAddr)))
					memcpy((void*)&(app->DefaultIPAddr), (void*)&(app->MyIPAddr), sizeof(IP_ADDR));
				else
					curHTTP.smPost = SM_CFG_FAILURE;
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"gw"))
			{// Read new gateway address
				if(!StringToIPAddress(curHTTP.data+8, &(app->MyGateway)))
					curHTTP.smPost = SM_CFG_FAILURE;
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"subnet"))
			{// Read new static subnet
				if(StringToIPAddress(curHTTP.data+8, &(app->MyMask)))
					memcpy((void*)&(app->DefaultMask), (void*)&(app->MyMask), sizeof(IP_ADDR));
				else
					curHTTP.smPost = SM_CFG_FAILURE;
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"dns1"))
			{// Read new primary DNS server
				if(!StringToIPAddress(curHTTP.data+8, &(app->PrimaryDNSServer)))
					curHTTP.smPost = SM_CFG_FAILURE;
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"dns2"))
			{// Read new secondary DNS server
				if(!StringToIPAddress(curHTTP.data+8, &(app->SecondaryDNSServer)))
					curHTTP.smPost = SM_CFG_FAILURE;
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"mac"))
			{// Read new MAC address
				WORD_VAL w;
				BYTE i;
	
				ptr = curHTTP.data+8;
	
				for(i = 0; i < 12; i++)
				{// Read the MAC address
					
					// Skip non-hex bytes
					while( *ptr != 0x00 && !(*ptr >= '0' && *ptr <= '9') && !(*ptr >= 'A' && *ptr <= 'F') && !(*ptr >= 'a' && *ptr <= 'f') )
						ptr++;
	
					// MAC string is over, so zeroize the rest
					if(*ptr == 0x00)
					{
						for(; i < 12; i++)
							curHTTP.data[i] = '0';
						break;
					}
					
					// Save the MAC byte
					curHTTP.data[i] = *ptr++;
				}
				
				// Read MAC Address, one byte at a time
				for(i = 0; i < 6; i++)
				{				
					w.v[1] = curHTTP.data[i*2];
					w.v[0] = curHTTP.data[i*2+1];
					app->MyMACAddr.v[i] = hexatob(w);
				}
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"host"))
			{// Read new hostname
				for(len = strlen((char*)&curHTTP.data[8]); len < 15; len++)
					curHTTP.data[8+len] = ' ';
				memcpy((void*)app->NetBIOSName, (void*)curHTTP.data+8, 15);
				app->NetBIOSName[15] = 0;
				strupr((char*)app->NetBIOSName);
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"dhcpen"))
			{// Read new DHCP Enabled flag
				if(curHTTP.data[8] == '1')
					app->Flags.bIsDHCPEnabled = 1;
				else
					app->Flags.bIsDHCPEnabled = 0;
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"pass"))
			{// Read new password
				for(len = strlen((char*)&curHTTP.data[8]); len < 8; len++)
					curHTTP.data[8+len] = 0;
				memcpy((void*)app->Password, (void*)curHTTP.data+8, 8);
				app->Password[8] = 0;
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"tzone"))
			{// Read time zone
				app->TZ = atoi((char*)&curHTTP.data[8]);
			}
			else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"utcs"))
			{// Read the time server param
				for(len = strlen((char*)&curHTTP.data[8]); len < 20; len++)
					curHTTP.data[8+len] = 0;
				memcpy((void*)app->UTC_Serv, (void*)curHTTP.data+8, 20);
				app->UTC_Serv[20] = 0;
			}
			break;
			
		case SM_CFG_SUCCESS:
			// Save new settings and force a reboot
			
			// If DCHP, then disallow editing of DefaultIP and DefaultMask
			if(app->Flags.bIsDHCPEnabled)
			{
				// If DHCP is enabled, then reset the default IP and mask
				app->DefaultIPAddr.v[0] = MY_DEFAULT_IP_ADDR_BYTE1;
				app->DefaultIPAddr.v[1] = MY_DEFAULT_IP_ADDR_BYTE2;
				app->DefaultIPAddr.v[2] = MY_DEFAULT_IP_ADDR_BYTE3;
				app->DefaultIPAddr.v[3] = MY_DEFAULT_IP_ADDR_BYTE4;
				app->DefaultMask.v[0] 	= MY_DEFAULT_MASK_BYTE1;
				app->DefaultMask.v[1] 	= MY_DEFAULT_MASK_BYTE2;
				app->DefaultMask.v[2] 	= MY_DEFAULT_MASK_BYTE3;
				app->DefaultMask.v[3] 	= MY_DEFAULT_MASK_BYTE4;
			}

			ptr = (BYTE*)app;
		    XEEBeginWrite(0x0000);
		    XEEWrite(0x61);
		    for (len = 0; len < sizeof(AppConfig); len++)
		        XEEWrite(*ptr++);
		    XEEEndWrite();
	        while(XEEIsBusy());
			

			// Set the board to reboot to the new address
			strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/protect/reboot.htm?");
			memcpy((void*)(curHTTP.data+20), (void*)app->NetBIOSName, 16);
			ptr = curHTTP.data;
			while(*ptr != ' ' && *ptr != '\0')
				ptr++;
			*ptr = '\0';
			curHTTP.httpStatus = HTTP_REDIRECT;			
			return HTTP_IO_DONE;
			
		case SM_CFG_FAILURE:
			// An error occurred parsing the IP, so don't save 
			// anything to prevent network errors.		
			strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/protect/config_error.htm");
			curHTTP.httpStatus = HTTP_REDIRECT;		
			return HTTP_IO_DONE;			
	}

	return HTTP_IO_WAITING;		// Assume we're waiting to process more data
}

/*****************************************************************************
  Function:
	static HTTP_IO_RESULT HTTPPostDefault(void)

  Summary:
	Processes the factory defaults form on protect/defaults.htm

  Return Values:
  	HTTP_IO_DONE - all parameters have been processed
  	HTTP_IO_WAITING - the function is pausing to continue later
  	HTTP_IO_NEED_DATA - data needed by this function has not yet arrived
  ***************************************************************************/
static HTTP_IO_RESULT HTTPPostDefault(void)
{
	#define SM_DEF_START		(0u)
	#define SM_DEF_READ_NAME	(1u)
	#define SM_DEF_READ_VALUE	(2u)
	#define SM_DEF_SUCCESS		(3u)
	BYTE *ptr;

	switch(curHTTP.smPost)
	{
		case SM_DEF_START:
			// Move to next state, but no break
			curHTTP.smPost = SM_DEF_READ_NAME;
			
		case SM_DEF_READ_NAME:
			// If all parameters have been read, end
			if(curHTTP.byteCount == 0)
			{
				curHTTP.smPost = SM_DEF_SUCCESS;
				break;
			}		
			// Read a name
			if(HTTPReadPostName(curHTTP.data, 8) == HTTP_READ_INCOMPLETE)
				return HTTP_IO_NEED_DATA;				
			// Move to reading a value, but no break
			curHTTP.smPost = SM_DEF_READ_VALUE;
			
		case SM_DEF_READ_VALUE:
			// Read a value
			if(HTTPReadPostValue(curHTTP.data + 8, 24) == HTTP_READ_INCOMPLETE)
				return HTTP_IO_NEED_DATA;		
			// Default action after this is to read the name, unless there's an error
			curHTTP.smPost = SM_DEF_READ_NAME;
			break;
			
		case SM_CFG_SUCCESS:
			// Load the factory defaults on next reboot
			//Disable for debug
		    XEEBeginWrite(0x0000);
		    XEEWrite(0xFF);
		    XEEEndWrite();


			// Set the board to reboot to the new address
			strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/protect/reboot.htm?");
			strcpypgm2ram((char*)curHTTP.data+20, (ROM char*)MY_DEFAULT_HOST_NAME);
			ptr = curHTTP.data;
			while(*ptr != ' ' && *ptr != '\0')
				ptr++;
			*ptr = '\0';
			curHTTP.httpStatus = HTTP_REDIRECT;				
			return HTTP_IO_DONE;					
	}
	return HTTP_IO_WAITING;		// Assume we're waiting to process more data
}

/*****************************************************************************
  Function:
	static HTTP_IO_RESULT HTTPPostReboot(void)

  Summary:
	Processes the factory defaults form on protect/defaults.htm

  Return Values:
  	HTTP_IO_DONE - all parameters have been processed
  	HTTP_IO_WAITING - the function is pausing to continue later
  	HTTP_IO_NEED_DATA - data needed by this function has not yet arrived
  ***************************************************************************/
static HTTP_IO_RESULT HTTPPostReboot(void)
{
	#define SM_RES_START		(0u)
	#define SM_RES_READ_NAME	(1u)
	#define SM_RES_READ_VALUE	(2u)
	#define SM_RES_SUCCESS		(3u)
	BYTE *ptr;

	switch(curHTTP.smPost)
	{
		case SM_RES_START:
			// Move to next state, but no break
			curHTTP.smPost = SM_RES_READ_NAME;
			
		case SM_RES_READ_NAME:
			// If all parameters have been read, end
			if(curHTTP.byteCount == 0)
			{
				curHTTP.smPost = SM_RES_SUCCESS;
				break;
			}		
			// Read a name
			if(HTTPReadPostName(curHTTP.data, 8) == HTTP_READ_INCOMPLETE)
				return HTTP_IO_NEED_DATA;				
			// Move to reading a value, but no break
			curHTTP.smPost = SM_RES_READ_VALUE;
			
		case SM_RES_READ_VALUE:
			// Read a value
			if(HTTPReadPostValue(curHTTP.data + 8, 24) == HTTP_READ_INCOMPLETE)
				return HTTP_IO_NEED_DATA;		
			// Default action after this is to read the name, unless there's an error
			curHTTP.smPost = SM_RES_READ_NAME;
			break;
			
		case SM_RES_SUCCESS:
			// Set the board to reboot to the new address
			strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/protect/reset.htm?");
			strcpypgm2ram((char*)curHTTP.data+19, (ROM char*)MY_DEFAULT_HOST_NAME);
			ptr = curHTTP.data;
			while(*ptr != ' ' && *ptr != '\0')
				ptr++;
			*ptr = '\0';
			curHTTP.httpStatus = HTTP_REDIRECT;				
			return HTTP_IO_DONE;					
	}
	return HTTP_IO_WAITING;		// Assume we're waiting to process more data
}


/****************************************************************************
  Section:
	Dynamic Variable Callback Functions
  ***************************************************************************/

/*****************************************************************************
  Function:
	void HTTPPrint_varname(void)
	
  Internal:
  	See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/

// Prints the build date to the WEB page
void HTTPPrint_builddate(void)
{
	curHTTP.callbackPos = 0x01;
	if(TCPIsPutReady(sktHTTP) < strlenpgm((ROM char*)__DATE__))
		return;
	
	curHTTP.callbackPos = 0x00;
	TCPPutROMString(sktHTTP, (ROM void*)__DATE__);
}

// Prints the firmware release in WEB page
void HTTPPrint_version(void)
{
	TCPPutROMString(sktHTTP, (ROM void*)FW_REL);
}

/*******************************************
** Determine and print the current connection with the script page  
********************************************/
void HTTPPrint_conn_stat(void)
{
	if (conn_flag == 0)
		TCPPutROMString(sktHTTP, (ROM void*)"Successful");
	else
		TCPPutROMString(sktHTTP, (ROM void*)"Not successful");
}

/*******************************************
** Determine and print the current time 
*********************************************/
void HTTPPrint_utc_time(void)
{
	BYTE *UTCString;
	UTCString = ctime(&sysTime);
   	TCPPutString(sktHTTP, UTCString);
}

/*******************************************
** Determine and print the current date 
*********************************************/
void HTTPPrint_utc_date(void)
{
	BYTE *UTCString;
	UTCString = cdate(&sysTime);
   	TCPPutString(sktHTTP, UTCString);
}

/*******************************************
** Determine and print the current time zone
*********************************************/
void HTTPPrint_utc_zone(void)
{
	BYTE UTCString[5];

	if (GetTimeZone() > 0)
		TCPPutROMString(sktHTTP, (ROM void*)"+");
    itoa(GetTimeZone(), (BYTE*)UTCString);	
   	TCPPutString(sktHTTP, UTCString);
}

/*******************************************
** Determine and print the current sync status of NTP  
********************************************/
void HTTPPrint_utc_stat(void)
{
	if (SNTP_for_the_first_time != 0)
		TCPPutROMString(sktHTTP, (ROM void*)"Syncronized");
	else
		TCPPutROMString(sktHTTP, (ROM void*)"Not syncronized");
}


/*******************************************
** Print the NTP server address  
********************************************/
void HTTPPrint_utc_server(void)
{
	curHTTP.callbackPos = 0x01;
	if(TCPIsPutReady(sktHTTP) < strlen((char*)AppConfig.UTC_Serv))
		return;
	
	curHTTP.callbackPos = 0x00;
	TCPPutString(sktHTTP, AppConfig.UTC_Serv);
	return;
}

/*******************************************
** Determine and print the time zone menu 
********************************************/
void HTTPPrint_tzone_val(void)
{
	BYTE UTCString[5];
	int i_cnt;

	// Initiate that the process is not finished
	if (curHTTP.callbackPos == 0)
		curHTTP.callbackPos = 27;

	// Make sure there's enough output space
	if(TCPIsPutReady(sktHTTP) < 48u)
		return;

	// calculate the index to be written
	i_cnt = curHTTP.callbackPos - 13;

	// write the choice options
	TCPPutROMString(sktHTTP, (ROM void*)"<option value=\"");
	itoa(i_cnt, (BYTE*)UTCString);
	TCPPutString(sktHTTP, UTCString);
	TCPPutROMString(sktHTTP, (ROM void*)"\"");
	// write the selected option
	if (i_cnt==GetTimeZone())
		TCPPutROMString(sktHTTP, (ROM void*)" selected");
	TCPPutROMString(sktHTTP, (ROM void*)">");
	// write the text for the options
 	TCPPutString(sktHTTP, UTCString);
	TCPPutROMString(sktHTTP, (ROM void*)"</option>");

	curHTTP.callbackPos--;	
}

/*******************************************
** Print the script server address  
********************************************/
void HTTPPrint_config_scr_srv(void)
{
	curHTTP.callbackPos = 0x01;
	if(TCPIsPutReady(sktHTTP) < strlen((char*)AppConfig.Scr_Svr))
		return;
	
	curHTTP.callbackPos = 0x00;
	TCPPutString(sktHTTP, AppConfig.Scr_Svr);
	return;
}

/*******************************************
** Print the script local path 
********************************************/
void HTTPPrint_config_scr_path(void)
{
	curHTTP.callbackPos = 0x01;
	if(TCPIsPutReady(sktHTTP) < strlen((char*)AppConfig.Scr_Path))
		return;
	
	curHTTP.callbackPos = 0x00;
	TCPPutString(sktHTTP, AppConfig.Scr_Path);
	return;
}

/*******************************************
** Determine and print the script refresh rate 
*********************************************/
void HTTPPrint_config_scr_ref(void)
{
	BYTE *UTCString;
	ultoa(AppConfig.Scr_Refresh, (BYTE*)UTCString);
   	TCPPutString(sktHTTP, UTCString);
}

/*******************************************
** Determine and print the UP-TIME 
*********************************************/
void HTTPPrint_uptime(void)
{
	DWORD uDays, uHours, uMins, uSecs; 
	BYTE UTCString[8];

	// Calculating exact seconds
	uMins = upTime / 60;
	uSecs = upTime - uMins*60;
	// Calculating exact minutes
	uHours = uMins / 60;
	uMins = uMins - uHours*60;
	// Calculating exaxt hours
	uDays = uHours / 24;
	uHours = uHours - uDays*24;

	if (uDays != 0)
	{
   		uitoa(uDays, (BYTE*)UTCString);
		TCPPutString(sktHTTP, UTCString);
		TCPPutROMString(sktHTTP, (ROM void*)" days, ");
	}
	if (uHours != 0)
	{
   		uitoa(uHours, (BYTE*)UTCString);
		TCPPutString(sktHTTP, UTCString);
		TCPPutROMString(sktHTTP, (ROM void*)" hours, ");
	}
	if (uMins != 0)
	{
   		uitoa(uMins, (BYTE*)UTCString);
		TCPPutString(sktHTTP, UTCString);
		TCPPutROMString(sktHTTP, (ROM void*)" min., ");
	}
   	uitoa(uSecs, (BYTE*)UTCString);
	TCPPutString(sktHTTP, UTCString);
	TCPPutROMString(sktHTTP, (ROM void*)" sec. ");
}

// Prints the hostname
void HTTPPrint_hellomsg(void)
{
	TCPPutString(sktHTTP, AppConfig.NetBIOSName);
	return;
}

// Prints the current password in the CONFIG PAGE
void HTTPPrint_config_password(void)
{
	TCPPutString(sktHTTP, AppConfig.Password);
	return;
}

// Prints the current UTC server in CONFIG PAGE
void HTTPPrint_config_utc_server(void)
{
	TCPPutString(sktHTTP, AppConfig.UTC_Serv);
	return;
}

// Prints the current IP in WEB pages
void HTTPPrintIP(IP_ADDR ip)
{
	BYTE digits[4];
	BYTE i;
	
	for(i = 0; i < 4u; i++)
	{
		if(i)
			TCPPut(sktHTTP, '.');
		uitoa(ip.v[i], digits);
		TCPPutString(sktHTTP, digits);
	}
}

// Prints the current hostname in WEB pages
void HTTPPrint_config_hostname(void)
{
	TCPPutString(sktHTTP, AppConfig.NetBIOSName);
	return;
}

// Prints the DHCP check box status in WEB pages
void HTTPPrint_config_dhcpchecked(void)
{
	if(AppConfig.Flags.bIsDHCPEnabled)
		TCPPutROMString(sktHTTP, (ROM BYTE*)"checked");
	return;
}

void HTTPPrint_config_ip(void)
{
	HTTPPrintIP(AppConfig.MyIPAddr);
	return;
}

void HTTPPrint_config_gw(void)
{
	HTTPPrintIP(AppConfig.MyGateway);
	return;
}

void HTTPPrint_config_subnet(void)
{
	HTTPPrintIP(AppConfig.MyMask);
	return;
}

void HTTPPrint_config_dns1(void)
{
	HTTPPrintIP(AppConfig.PrimaryDNSServer);
	return;
}

void HTTPPrint_config_dns2(void)
{
	HTTPPrintIP(AppConfig.SecondaryDNSServer);
	return;
}

// Display the MAC address in config page
void HTTPPrint_config_mac(void)
{
	BYTE i;
	
	if(TCPIsPutReady(sktHTTP) < 18u)
	{//need 17 bytes to write a MAC
		curHTTP.callbackPos = 0x01;
		return;
	}	
	
	// Write each byte
	for(i = 0; i < 6u; i++)
	{
		//if(i)
		//TCPPut(sktHTTP, ':');
		TCPPut(sktHTTP, btohexa_high(AppConfig.MyMACAddr.v[i]));
		TCPPut(sktHTTP, btohexa_low(AppConfig.MyMACAddr.v[i]));
	}
	
	// Indicate that we're done
	curHTTP.callbackPos = 0x00;
	return;
}

// Display the MAC address in main page
void HTTPPrint_mac_addr(void)
{
	BYTE i;
	
	if(TCPIsPutReady(sktHTTP) < 18u)
	{//need 17 bytes to write a MAC
		curHTTP.callbackPos = 0x01;
		return;
	}	
	
	// Write each byte
	for(i = 0; i < 6u; i++)
	{
		if(i)
		TCPPut(sktHTTP, ':');
		TCPPut(sktHTTP, btohexa_high(AppConfig.MyMACAddr.v[i]));
		TCPPut(sktHTTP, btohexa_low(AppConfig.MyMACAddr.v[i]));
	}
	
	// Indicate that we're done
	curHTTP.callbackPos = 0x00;
	return;
}

void HTTPPrint_reboot(void)
{
	// This is not so much a print function, but causes the board to reboot
	// when the configuration is changed.  If called via an AJAX call, this
	// will gracefully reset the board and bring it back online immediately
	Reset();
}

void HTTPPrint_rebootaddr(void)
{
	// This is the expected address of the board upon rebooting
	TCPPutString(sktHTTP, curHTTP.data);	
}

#endif
